"""add ReleaseMonitor.last_torrent_search_stamp

Revision ID: 2ee6e71eb2f
Revises: 36bd688b04b
Create Date: 2014-12-16 18:31:20.327836

"""

# revision identifiers, used by Alembic.
revision = '2ee6e71eb2f'
down_revision = '36bd688b04b'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release_monitor', sa.Column('last_torrent_search_stamp', sa.Integer(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_column('release_monitor', 'last_torrent_search_stamp')
    ### end Alembic commands ###
