"""add search_name to Release and Show

Revision ID: 218daaf29e40
Revises: 18b0a0874c5a
Create Date: 2016-11-15 20:00:04.915748

"""

# revision identifiers, used by Alembic.
revision = '218daaf29e40'
down_revision = '18b0a0874c5a'

from alembic import op
import sqlalchemy as sa


def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.add_column('release', sa.Column('search_name', sa.String(), nullable=True))
    op.add_column('show', sa.Column('search_name', sa.String(), nullable=True))
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    pass
    ### end Alembic commands ###
